import json
import time
import requests


def get_VM_backups(base_url, token, VMid, VCid, latest=False):
    url = base_url + '/api/backups/vmware/vms'
    filter = {
        "vcenter_id": {"$eq": VCid},
        "vm_id": {"$eq": VMid}
    }
    backups_response = http_get(url + '?limit=500&filter=' + json.dumps(filter),
                                token=token)
    if backups_response.status_code != 200 or \
            len(backups_response.json()["_embedded"]["items"]) == 0:
        raise Exception("Backup could not be found.")
    if latest:
        pit_id = backups_response.json()['_embedded']['items'][0]['id']
        return pit_id
    else:
        return backups_response.json()


def get_PIT_params(base_url, token, pit_id):
    url = base_url + '/api/backups/vmware/vms/' + pit_id
    time.sleep(3)
    response = http_get(url, token=token)
    return response.json()


def recoverVM(base_url, token, body, netON=False, powerON=False):
    url = base_url + '/api/restores/vmware/vms'
    response = http_post(url, token, body=body)
    return response.json()


def listVCs(base_url, token):
    url = base_url + '/api/datasources/vmware/vcenters'
    reponse = http_get(url, token=token)
    return reponse.json()


def listVMs(base_url, token, VCid):
    url = base_url + '/api/datasources/vmware/vcenters/' + \
        str(VCid) + '/vms?limit=50'
    reponse = http_get(url, token=token)
    return reponse.json()


def get_token(base_url, email, pw):
    url = base_url + '/api/authentication/tokens'
    print("Attempting login.")
    login_response = http_post(url, body={'email': email, 'password': pw})
    return login_response.json()['token']


def http_get(url, token=None):
    REQUEST_HEADERS = {'Accept': "application/api.clumio.*=v0+json"}
    if token:
        REQUEST_HEADERS['authorization'] = "Bearer " + token
    response = requests.get(url, headers=REQUEST_HEADERS)
    time.sleep(1)
    if response.status_code != 200:
        print('Error: %s' % response.text)
    return response


def http_post(url, token=None, body=None, patch=False):
    REQUEST_HEADERS = {'Accept': "application/api.clumio.*=v0+json"}
    if token:
        REQUEST_HEADERS['authorization'] = "Bearer " + token

    if body and not isinstance(body, str):
        body = json.dumps(body)

    if not patch:
        response = requests.post(url, headers=REQUEST_HEADERS, data=body)
    else:
        response = requests.patch(url, headers=REQUEST_HEADERS, data=body)
    time.sleep(1)
    if response.status_code == 200:
        print('Success!\n\n')
    else:
        print('Error: %s' % response.text)
    return response
